/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 rlGets is a replacement for fgets(3) that uses the GNU readline
 library (and that do NOT use the GNU history library).  The synopsis
 is:

 char *rlGets(const char *prompt, char *buffer, size_t size, int trim)

 rlGets writes "prompt" on the standard output stream, unless "prompt"
 is the NULL pointer or points to an empty string; then reads, from
 the standard input stream, at most one less than "size" characters,
 and stores them into the array "buffer".  A zero character ('\0') is
 always stored after the last character in "buffer".

 If the formal parameter "trim" is not 0, the leading and the trailing
 whitespace of the input line is removed.

 rlGets returns "buffer" on success; and the NULL pointer when an
 input error is detected, or when the end-of-file has been read.

 *------------------------------------------------------------------*
 $Id: rlGet.c,v 1.1 2004/02/28 16:52:50 loreti Exp $
 *------------------------------------------------------------------*/

#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <readline/readline.h>
#include <readline/history.h>

char *rlGets(
  const char *prompt,
  char       *buffer,
  size_t      size,
  int         trim
) {

/**
 | rlGets is a replacement for fgets(3) that uses the GNU readline
 | library (and that do NOT use the GNU history library).
 |
 | rlGets writes "prompt" on the standard output stream, unless
 | "prompt" is the NULL pointer or points to an empty string; then
 | reads, from the standard input stream, at most one less than "size"
 | characters, and stores them into the array "buffer".  A zero
 | character ('\0') is always stored after the last character in
 | "buffer".
 |
 | If the formal parameter "trim" is not 0, the leading and the
 | trailing whitespace of the input line is removed.
 |
 |  rlGets returns "buffer" on success; and the NULL pointer when an
 | input error is detected, or when the end-of-file has been read.
**/

  char *line;

  assert(size > 1);
  if ((line = readline(prompt)) == 0) return 0;

  if (trim) {
    char c, *start, *pc, *last;
    size_t n;

    /**
     | Removes from "line" the leading whitespace, saving in "start" a
     | pointer to the first valid character; if the input contains
     | only whitespace, returns.
    **/

    for (start = line;   ;   start++) {
      if ((c = *start) == '\0') {
        *buffer = '\0';
        free(line);
        return buffer;
      }
      if (!isspace((unsigned char) c)) break;
    }

    /**
     | Looks ahead for the last non-whitespace character, saving in
     | "last" a pointer to the character after the last valid one.
    **/

    for (pc = last = start+1;   (c = *pc++) != '\0';   ) {
      if (!isspace((unsigned char) c)) last = pc;
    }

    /**
     | Removes the trailing whitespace, and files the trimmed buffer
     | into the history ring.
    **/

    *last = '\0';
    add_history(start);

    /**
     | Moves up to "size" characters (the trailing \0 included) into
     | the program buffer.
    **/

    if ((n = last - start) >= size) {
      n = size - 1;
      *(start + n) = '\0';
    }
    memcpy(buffer, start, n+1);

  } else {
    char *from = line;

    /**
     | We do not have to remove leading and trailing whitespace: so,
     | the original line (if not empty) is filed into the history
     | ring; and up to "size" characters are moved from "line" to
     | "buffer".
    **/

    if (*line) add_history(line);
    while (--size  &&  (*buffer++ = *from++)) { }
    *buffer = '\0';
  }

  /**
   | The buffer allocated from the readline library is released, and
   | "buffer" returned.
  **/

  free(line);
  return buffer;
}
